@@ -158,7 +158,8 @@
           (format file " ~10a |" lisp)))
       (terpri file)
       (format file "~a~%" line)
-      (dolist (dh-name '(:curve25519 :curve448 :elgamal))
+      (dolist (dh-name '(:curve25519 :curve448 :elgamal
+                         :secp256k1 :secp256r1 :secp384r1 :secp521r1))
         (format file "| ~14a |" dh-name)
         (dolist (implementation *lisp-implementations*)
           (let* ((lisp (car implementation))
@@ -205,7 +206,8 @@
           (format file " ~10a |" lisp)))
       (terpri file)
       (format file "~a~%" line)
-      (dolist (signature-name '(:dsa :ed25519 :ed448 :elgamal :rsa))
+      (dolist (signature-name '(:dsa :ed25519 :ed448 :elgamal :rsa
+                                :secp256k1 :secp256r1 :secp384r1 :secp521r1))
         (format file "| ~14a |" signature-name)
         (dolist (implementation *lisp-implementations*)
           (let* ((lisp (car implementation))
@@ -229,7 +231,8 @@
           (format file " ~10a |" lisp)))
       (terpri file)
       (format file "~a~%" line)
-      (dolist (signature-name '(:dsa :ed25519 :ed448 :elgamal :rsa))
+      (dolist (signature-name '(:dsa :ed25519 :ed448 :elgamal :rsa
+                                :secp256k1 :secp256r1 :secp384r1 :secp521r1))
         (format file "| ~14a |" signature-name)
         (dolist (implementation *lisp-implementations*)
           (let* ((lisp (car implementation))
