@@ -100,10 +100,9 @@
 
 (rtest:deftest digests.crypto-package
   (every #'(lambda (s)
-             (and (eq (symbol-package s) (find-package :ironclad))
-                  (eq (nth-value 1 (find-symbol (symbol-name s)
-                                                (find-package :ironclad)))
-                      :external)))
+             (eq (nth-value 1 (find-symbol (symbol-name s)
+                                           (find-package :ironclad)))
+                 :external))
          (crypto:list-all-digests))
   t)
 
